-----------------------------------------------------------------------------------------------
-- Client Lua Script for Translator. By lyravega, Phare@Ravenous
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "ChatSystemLib"
 
local Translator, messagesOnHold, delay = {}, {}, 0.1
 
function Translator:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
    
    o.isEnabled = true

    return o
end

function Translator:Init()
	Apollo.RegisterAddon(self)
	Apollo.RegisterSlashCommand("translator", "OnToggleTranslator", self)
	Apollo.RegisterEventHandler("ChatMessage", "OnChatMessage", self)
	Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
	Apollo.RegisterEventHandler("ToggleTranslator", "OnToggleTranslator", self)
	self.delayer = ApolloTimer.Create(delay, true, "Delayer", self); self.delayer:Start()
end

function Translator:OnSave(saveDepth)
	if saveDepth ~= GameLib.CodeEnumAddonSaveLevel.Character then return nil end

	local savedVariables = {}
	savedVariables.isEnabled = self.isEnabled

	return savedVariables
end

function Translator:OnRestore(saveDepth, savedVariables)
	if saveDepth ~= GameLib.CodeEnumAddonSaveLevel.Character then return end	
	if savedVariables == {} or savedVariables == nil then return end
	
	self.isEnabled = savedVariables.isEnabled
	self.hasRolledCredits = savedVariables.hasRolledCredits
end

function Translator:OnChatMessage(channelCurrent, tMessage)
	if not self.isEnabled then return end
	if not tMessage.bCrossFaction then return end
	
	messagesOnHold[os.clock()+delay] = {tMessage.arMessageSegments[1].strText, tMessage.strSender}
end

function Translator:Delayer()
	local osclock = os.clock()
	
	for message, send in next, messagesOnHold do
		if message < osclock then ChatSystemLib.PostOnChannel( ChatSystemLib.ChatChannel_Datachron, send[1], send[2] ); messagesOnHold[message] = nil end
	end
end

function Translator:OnInterfaceMenuListHasLoaded()
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "Translator", {"ToggleTranslator", "", "IconSprites:Icon_MapNode_Map_HousingTransit"})
	self:UpdateInterfaceMenuAlerts()
end

function Translator:UpdateInterfaceMenuAlerts()
	local credits = "by lyravega, Phare@Ravenous\n"
	local status = self.isEnabled and "Enabled" or "Disabled"
	
	Event_FireGenericEvent("InterfaceMenuList_AlertAddOn", "Translator", {self.isEnabled, credits..status, 0})
end

function Translator:OnToggleTranslator()	
	self.isEnabled = not self.isEnabled
	self:UpdateInterfaceMenuAlerts()
	
	local message = self.isEnabled and "Activated" or "Deactivated"
	
	ChatSystemLib.PostOnChannel( ChatSystemLib.ChatChannel_Datachron, message, "Translator" )
end

local TranslatorInst = Translator:new()
TranslatorInst:Init()